@@ -17,10 +17,6 @@
 #include <KLocalizedString> // KLocalizedString::localizedFilePath. Need such functionality in, hmm, QLocale? QStandardPaths?
 #include <KSharedConfig>
 
-#ifdef WITH_BREEZEICONS_LIB
-#include <BreezeIcons>
-#endif
-
 #include <QAction>
 #include <QCoreApplication>
 #include <QDebug>
@@ -40,61 +36,80 @@
 
 Q_GLOBAL_STATIC(QString, _themeOverride)
 
-// Support for icon themes in RCC files.
-// The intended use case is standalone apps on Windows / MacOS / etc.
-// For this reason we use AppDataLocation: BINDIR/data on Windows, Resources on OS X
-void initRCCIconTheme()
+#ifdef Q_OS_ANDROID
+
+// on Android we are marked as porting AID, don't mess with theming
+void KIconTheme::initTheme()
 {
-#ifdef WITH_BREEZEICONS_LIB
-    BreezeIcons::initIcons();
+}
+
 #else
-    const QString iconThemeRcc = QStandardPaths::locate(QStandardPaths::AppDataLocation, QStringLiteral("icontheme.rcc"));
-    if (!iconThemeRcc.isEmpty()) {
-        const QString iconThemeName = QStringLiteral("kf6_rcc_theme");
-        const QString iconSubdir = QStringLiteral("/icons/") + iconThemeName;
-        if (QResource::registerResource(iconThemeRcc, iconSubdir)) {
-            if (QFileInfo::exists(QLatin1Char(':') + iconSubdir + QStringLiteral("/index.theme"))) {
-                // Tell Qt about the theme
-                // Note that since qtbase commit a8621a3f8, this means the QPA (i.e. KIconLoader) will NOT be used.
-                QIcon::setThemeName(iconThemeName); // Qt looks under :/icons automatically
-                // Tell KIconTheme about the theme, in case KIconLoader is used directly
-                *_themeOverride() = iconThemeName;
-            } else {
-                qWarning() << "No index.theme found in" << iconThemeRcc;
-                QResource::unregisterResource(iconThemeRcc, iconSubdir);
-            }
-        } else {
-            qWarning() << "Invalid rcc file" << iconThemeRcc;
-        }
-    }
-#endif
-}
-Q_COREAPP_STARTUP_FUNCTION(initRCCIconTheme)
 
-// Makes sure the icon theme fallback is set to breeze or one of its
-// variants. Most of our apps use "lots" of icons that most of the times
-// are only available with breeze, we still honour the user icon theme
-// but if the icon is not found there, we go to breeze since it's almost
-// sure it'll be there
-static void setBreezeFallback()
+#include <BreezeIcons>
+
+// do init only once and avoid later helpers to mess with it again
+static bool initThemeUsed = false;
+
+// startup function to set theme once the app got constructed
+static void initThemeHelper()
 {
-    if (const QPlatformTheme *theme = QGuiApplicationPrivate::platformTheme()) {
-        const QVariant themeHint = theme->themeHint(QPlatformTheme::SystemIconFallbackThemeName);
-        if (themeHint.isValid()) {
-            const QString iconTheme = themeHint.toString();
-            if (iconTheme.contains(QStringLiteral("breeze"), Qt::CaseInsensitive)) {
-                QIcon::setFallbackThemeName(iconTheme);
-                return;
+    // Makes sure the icon theme fallback is set to breeze or one of its
+    // variants. Most of our apps use "lots" of icons that most of the times
+    // are only available with breeze, we still honour the user icon theme
+    // but if the icon is not found there, we go to breeze since it's almost
+    // sure it'll be there
+    BreezeIcons::initIcons();
+
+    // ensure lib call above did the job
+    Q_ASSERT(!QIcon::fallbackThemeName().isEmpty());
+
+    // only do further stuff if we requested it
+    if (!initThemeUsed) {
+        return;
+    }
+
+    // do nothing if we have the proper platform theme already
+    if (QGuiApplicationPrivate::platformTheme() && QGuiApplicationPrivate::platformTheme()->name() == QLatin1String("kde")) {
+        return;
+    }
+
+    // get config, with fallback to kdeglobals
+    const auto config = KSharedConfig::openConfig();
+
+    // TODO: add Qt API to really fully override the engine
+    // https://codereview.qt-project.org/c/qt/qtbase/+/559108
+
+    // enforce the theme configured by the user, with kdeglobals fallback
+    // if not set, use Breeze
+    QString themeToUse = KConfigGroup(config, "Icons").readEntry("Theme", QStringLiteral("breeze"));
+    QIcon::setThemeName(themeToUse);
+    // Tell KIconTheme about the theme, in case KIconLoader is used directly
+    *_themeOverride() = themeToUse;
+    qCDebug(KICONTHEMES) << "KIconTheme::initTheme() enforces the icon theme:" << themeToUse;
+}
+Q_COREAPP_STARTUP_FUNCTION(initThemeHelper)
+
+void KIconTheme::initTheme()
+{
+    // inject paths only once
+    if (!initThemeUsed) {
+        // inject our icon engine in the search path
+        // it will be used as the first found engine for a suffix will be taken
+        // this must be done before the QCoreApplication is constructed
+        const auto paths = QCoreApplication::libraryPaths();
+        for (const auto &path : paths) {
+            if (const QString ourPath = path + QStringLiteral("/kiconthemes6"); QFile::exists(ourPath)) {
+                QCoreApplication::addLibraryPath(ourPath);
             }
         }
     }
 
-    QIcon::setFallbackThemeName(QStringLiteral("breeze"));
+    // initThemeHelper will do the remaining work via Q_COREAPP_STARTUP_FUNCTION(initThemeHelper) above
+    initThemeUsed = true;
 }
 
-#ifndef Q_OS_ANDROID
-Q_COREAPP_STARTUP_FUNCTION(setBreezeFallback)
 #endif
+
 class KIconThemeDir;
 class KIconThemePrivate
 {
